@@ -1,6 +1,25 @@
+"""
+=============
+Looking Glass
+=============
+
+Example using mouse events to simulate a looking glass for inspecting data.
+
+.. note::
+    This example exercises the interactive capabilities of Matplotlib, and this
+    will not appear in the static documentation. Please run this code on your
+    machine to see the interactivity.
+
+    You can copy and paste individual parts, or download the entire example
+    using the link at the bottom of the page.
+"""
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.patches as patches
+
+# Fixing random state for reproducibility
+np.random.seed(19680801)
+
 x, y = np.random.rand(2, 200)
 
 fig, ax = plt.subplots()
@@ -10,17 +29,18 @@ ax.add_patch(circ)
 
 ax.plot(x, y, alpha=0.2)
 line, = ax.plot(x, y, alpha=1.0, clip_path=circ)
+ax.set_title("Left click and drag to move looking glass")
 
 
-class EventHandler(object):
+class EventHandler:
     def __init__(self):
-        fig.canvas.mpl_connect('button_press_event', self.onpress)
-        fig.canvas.mpl_connect('button_release_event', self.onrelease)
-        fig.canvas.mpl_connect('motion_notify_event', self.onmove)
+        fig.canvas.mpl_connect('button_press_event', self.on_press)
+        fig.canvas.mpl_connect('button_release_event', self.on_release)
+        fig.canvas.mpl_connect('motion_notify_event', self.on_move)
         self.x0, self.y0 = circ.center
         self.pressevent = None
 
-    def onpress(self, event):
+    def on_press(self, event):
         if event.inaxes != ax:
             return
 
@@ -29,11 +49,11 @@ class EventHandler(object):
 
         self.pressevent = event
 
-    def onrelease(self, event):
+    def on_release(self, event):
         self.pressevent = None
         self.x0, self.y0 = circ.center
 
-    def onmove(self, event):
+    def on_move(self, event):
         if self.pressevent is None or event.inaxes != self.pressevent.inaxes:
             return
 
