@@ -18,9 +18,9 @@ y = np.linspace(-2.0, 2.0, N)
 X, Y = np.meshgrid(x, y)
 
 # A low hump with a spike coming out.
-# Needs to have z/colour axis on a log scale so we see both hump and spike.
-# linear scale only shows the spike.
-Z1 = np.exp(-(X)**2 - (Y)**2)
+# Needs to have z/colour axis on a log scale, so we see both hump and spike.
+# A linear scale only shows the spike.
+Z1 = np.exp(-X**2 - Y**2)
 Z2 = np.exp(-(X * 10)**2 - (Y * 10)**2)
 z = Z1 + 50 * Z2
 
@@ -50,19 +50,12 @@ plt.show()
 
 #############################################################################
 #
-# ------------
+# .. admonition:: References
 #
-# References
-# """"""""""
+#    The use of the following functions, methods, classes and modules is shown
+#    in this example:
 #
-# The use of the following functions, methods and classes is shown
-# in this example:
-
-import matplotlib
-matplotlib.axes.Axes.contourf
-matplotlib.pyplot.contourf
-matplotlib.figure.Figure.colorbar
-matplotlib.pyplot.colorbar
-matplotlib.axes.Axes.legend
-matplotlib.pyplot.legend
-matplotlib.ticker.LogLocator
+#    - `matplotlib.axes.Axes.contourf` / `matplotlib.pyplot.contourf`
+#    - `matplotlib.figure.Figure.colorbar` / `matplotlib.pyplot.colorbar`
+#    - `matplotlib.axes.Axes.legend` / `matplotlib.pyplot.legend`
+#    - `matplotlib.ticker.LogLocator`
