@@ -7,7 +7,6 @@ Hatching capabilities for plotting histograms.
 """
 
 import itertools
-from collections import OrderedDict
 from functools import partial
 
 import numpy as np
@@ -21,8 +20,6 @@ def filled_hist(ax, edges, values, bottoms=None, orientation='v',
     """
     Draw a histogram as a stepped patch.
 
-    Extra kwargs are passed through to `fill_between`
-
     Parameters
     ----------
     ax : Axes
@@ -42,6 +39,9 @@ def filled_hist(ax, edges, values, bottoms=None, orientation='v',
        Orientation of the histogram.  'v' (default) has
        the bars increasing in the positive y-direction.
 
+    **kwargs
+        Extra keyword arguments are passed through to `.fill_between`.
+
     Returns
     -------
     ret : PolyCollection
@@ -49,16 +49,16 @@ def filled_hist(ax, edges, values, bottoms=None, orientation='v',
     """
     print(orientation)
     if orientation not in 'hv':
-        raise ValueError("orientation must be in {{'h', 'v'}} "
-                         "not {o}".format(o=orientation))
+        raise ValueError(f"orientation must be in {{'h', 'v'}} "
+                         f"not {orientation}")
 
     kwargs.setdefault('step', 'post')
+    kwargs.setdefault('alpha', 0.7)
     edges = np.asarray(edges)
     values = np.asarray(values)
     if len(edges) - 1 != len(values):
-        raise ValueError('Must provide one more bin edge than value not: '
-                         'len(edges): {lb} len(values): {lv}'.format(
-                             lb=len(edges), lv=len(values)))
+        raise ValueError(f'Must provide one more bin edge than value not: '
+                         f'{len(edges)=} {len(values)=}')
 
     if bottoms is None:
         bottoms = 0
@@ -86,7 +86,7 @@ def stack_hist(ax, stacked_data, sty_cycle, bottoms=None,
         The axes to add artists too
 
     stacked_data : array or Mapping
-        A (N, M) shaped array.  The first dimension will be iterated over to
+        A (M, N) shaped array.  The first dimension will be iterated over to
         compute histograms row-wise
 
     sty_cycle : Cycler or operable of dict
@@ -104,11 +104,11 @@ def stack_hist(ax, stacked_data, sty_cycle, bottoms=None,
 
         If not given and stacked data is an array defaults to 'default set {n}'
 
-        If stacked_data is a mapping, and labels is None, default to the keys
-        (which may come out in a random order).
+        If *stacked_data* is a mapping, and *labels* is None, default to the
+        keys.
 
-        If stacked_data is a mapping and labels is given then only
-        the columns listed by be plotted.
+        If *stacked_data* is a mapping and *labels* is given then only the
+        columns listed will be plotted.
 
     plot_func : callable, optional
         Function to call to draw the histogram must have signature:
@@ -117,9 +117,9 @@ def stack_hist(ax, stacked_data, sty_cycle, bottoms=None,
                           label=label, **kwargs)
 
     plot_kwargs : dict, optional
-        Any extra kwargs to pass through to the plotting function.  This
-        will be the same for all calls to the plotting function and will
-        over-ride the values in cycle.
+        Any extra keyword arguments to pass through to the plotting function.
+        This will be the same for all calls to the plotting function and will
+        override the values in *sty_cycle*.
 
     Returns
     -------
@@ -158,7 +158,7 @@ def stack_hist(ax, stacked_data, sty_cycle, bottoms=None,
     arts = {}
     for j, (data, label, sty) in loop_iter:
         if label is None:
-            label = 'dflt set {n}'.format(n=j)
+            label = f'dflt set {j}'
         label = sty.pop('label', label)
         vals, edges = hist_func(data)
         if bottoms is None:
@@ -181,14 +181,14 @@ hist_func = partial(np.histogram, bins=edges)
 
 # set up style cycles
 color_cycle = cycler(facecolor=plt.rcParams['axes.prop_cycle'][:4])
-label_cycle = cycler(label=['set {n}'.format(n=n) for n in range(4)])
+label_cycle = cycler(label=[f'set {n}' for n in range(4)])
 hatch_cycle = cycler(hatch=['/', '*', '+', '|'])
 
 # Fixing random state for reproducibility
 np.random.seed(19680801)
 
 stack_data = np.random.randn(4, 12250)
-dict_data = OrderedDict(zip((c['label'] for c in label_cycle), stack_data))
+dict_data = dict(zip((c['label'] for c in label_cycle), stack_data))
 
 ###############################################################################
 # Work with plain arrays
