@@ -1,33 +1,19 @@
 """
-============
-Multiprocess
-============
+===============
+Multiprocessing
+===============
 
 Demo of using multiprocessing for generating data in one process and
 plotting in another.
 
 Written by Robert Cimrman
 """
-from __future__ import print_function
 
+import multiprocessing as mp
 import time
-import numpy as np
-
-from multiprocessing import Process, Pipe
-
-# This example will likely not work with the native OSX backend.
-# Uncomment the following lines to use the qt5 backend instead.
-#
-# import matplotlib
-# matplotlib.use('qt5agg')
-#
-# Alternatively, with Python 3.4+ you may add the line
-#
-# import multiprocessing as mp; mp.set_start_method("forkserver")
-#
-# immediately after the ``if __name__ == "__main__"`` check.
 
 import matplotlib.pyplot as plt
+import numpy as np
 
 # Fixing random state for reproducibility
 np.random.seed(19680801)
@@ -41,7 +27,7 @@ np.random.seed(19680801)
 #
 
 
-class ProcessPlotter(object):
+class ProcessPlotter:
     def __init__(self):
         self.x = []
         self.y = []
@@ -89,15 +75,12 @@ class ProcessPlotter(object):
 #
 
 
-class NBPlot(object):
+class NBPlot:
     def __init__(self):
-        self.plot_pipe, plotter_pipe = Pipe()
+        self.plot_pipe, plotter_pipe = mp.Pipe()
         self.plotter = ProcessPlotter()
-        self.plot_process = Process(
-            target=self.plotter,
-            args=(plotter_pipe,)
-        )
-        self.plot_process.daemon = True
+        self.plot_process = mp.Process(
+            target=self.plotter, args=(plotter_pipe,), daemon=True)
         self.plot_process.start()
 
     def plot(self, finished=False):
@@ -111,11 +94,13 @@ class NBPlot(object):
 
 def main():
     pl = NBPlot()
-    for ii in range(10):
+    for _ in range(10):
         pl.plot()
         time.sleep(0.5)
     pl.plot(finished=True)
 
 
 if __name__ == '__main__':
+    if plt.get_backend() == "MacOSX":
+        mp.set_start_method("forkserver")
     main()
