@@ -70,15 +70,15 @@ class SliderGroup(Knob):
         self.sliderText = wx.TextCtrl(parent, -1, style=wx.TE_PROCESS_ENTER)
         self.slider = wx.Slider(parent, -1)
         # self.slider.SetMax(param.maximum*1000)
-        self.slider.SetRange(0, param.maximum * 1000)
+        self.slider.SetRange(0, int(param.maximum * 1000))
         self.setKnob(param.value)
 
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(self.sliderLabel, 0,
-                  wx.EXPAND | wx.ALIGN_CENTER | wx.ALL,
+                  wx.EXPAND | wx.ALL,
                   border=2)
         sizer.Add(self.sliderText, 0,
-                  wx.EXPAND | wx.ALIGN_CENTER | wx.ALL,
+                  wx.EXPAND | wx.ALL,
                   border=2)
         sizer.Add(self.slider, 1, wx.EXPAND)
         self.sizer = sizer
@@ -98,13 +98,13 @@ class SliderGroup(Knob):
         self.param.set(value)
 
     def setKnob(self, value):
-        self.sliderText.SetValue('%g' % value)
-        self.slider.SetValue(value * 1000)
+        self.sliderText.SetValue(f'{value:g}')
+        self.slider.SetValue(int(value * 1000))
 
 
 class FourierDemoFrame(wx.Frame):
     def __init__(self, *args, **kwargs):
-        wx.Frame.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         panel = wx.Panel(self)
 
         # create the GUI elements
@@ -115,9 +115,9 @@ class FourierDemoFrame(wx.Frame):
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer.Add(self.canvas, 1, wx.EXPAND)
         sizer.Add(self.frequencySliderGroup.sizer, 0,
-                  wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5)
+                  wx.EXPAND | wx.ALL, border=5)
         sizer.Add(self.amplitudeSliderGroup.sizer, 0,
-                  wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5)
+                  wx.EXPAND | wx.ALL, border=5)
         panel.SetSizer(sizer)
 
     def createCanvas(self, parent):
