@@ -1,6 +1,6 @@
 from collections import namedtuple
 import contextlib
-from functools import wraps
+from functools import lru_cache, wraps
 import inspect
 from inspect import Signature, Parameter
 import logging
@@ -21,6 +21,27 @@ from .transforms import (Bbox, IdentityTransform, Transform, TransformedBbox,
 _log = logging.getLogger(__name__)
 
 
+def _prevent_rasterization(draw):
+    # We assume that by default artists are not allowed to rasterize (unless
+    # its draw method is explicitly decorated). If it is being drawn after a
+    # rasterized artist and it has reached a raster_depth of 0, we stop
+    # rasterization so that it does not affect the behavior of normal artist
+    # (e.g., change in dpi).
+
+    @wraps(draw)
+    def draw_wrapper(artist, renderer, *args, **kwargs):
+        if renderer._raster_depth == 0 and renderer._rasterizing:
+            # Only stop when we are not in a rasterized parent
+            # and something has been rasterized since last stop.
+            renderer.stop_rasterizing()
+            renderer._rasterizing = False
+
+        return draw(artist, renderer, *args, **kwargs)
+
+    draw_wrapper._supports_rasterization = False
+    return draw_wrapper
+
+
 def allow_rasterization(draw):
     """
     Decorator for Artist.draw method. Provides routines
@@ -103,6 +124,15 @@ class Artist:
     zorder = 0
 
     def __init_subclass__(cls):
+
+        # Decorate draw() method so that all artists are able to stop
+        # rastrization when necessary. If the artist's draw method is already
+        # decorated (has a `_supports_rasterization` attribute), it won't be
+        # decorated.
+
+        if not hasattr(cls.draw, "_supports_rasterization"):
+            cls.draw = _prevent_rasterization(cls.draw)
+
         # Inject custom set() methods into the subclass with signature and
         # docstring based on the subclasses' properties.
 
@@ -168,7 +198,7 @@ class Artist:
         # Normally, artist classes need to be queried for mouseover info if and
         # only if they override get_cursor_data.
         self._mouseover = type(self).get_cursor_data != Artist.get_cursor_data
-        self._callbacks = cbook.CallbackRegistry()
+        self._callbacks = cbook.CallbackRegistry(signals=["pchanged"])
         try:
             self.axes = None
         except AttributeError:
@@ -186,7 +216,7 @@ class Artist:
     def __getstate__(self):
         d = self.__dict__.copy()
         # remove the unpicklable remove method, this will get re-added on load
-        # (by the axes) if the artist lives on an axes.
+        # (by the Axes) if the artist lives on an Axes.
         d['stale_callback'] = None
         return d
 
@@ -204,7 +234,7 @@ class Artist:
         Note: there is no support for removing the artist's legend entry.
         """
 
-        # There is no method to set the callback.  Instead the parent should
+        # There is no method to set the callback.  Instead, the parent should
         # set the _remove_method attribute directly.  This would be a
         # protected attribute if Python supported that sort of thing.  The
         # callback has one parameter, which is the child to be removed.
@@ -216,10 +246,8 @@ class Artist:
             if hasattr(self, 'axes') and self.axes:
                 # remove from the mouse hit list
                 self.axes._mouseover_set.discard(self)
-                # mark the axes as stale
                 self.axes.stale = True
-                # decouple the artist from the axes
-                self.axes = None
+                self.axes = None  # decouple the artist from the Axes
                 _ax_flag = True
 
             if self.figure:
@@ -238,13 +266,13 @@ class Artist:
     def have_units(self):
         """Return whether units are set on any axis."""
         ax = self.axes
-        return ax and any(axis.have_units() for axis in ax._get_axis_list())
+        return ax and any(axis.have_units() for axis in ax._axis_map.values())
 
     def convert_xunits(self, x):
         """
         Convert *x* using the unit type of the xaxis.
 
-        If the artist is not in contained in an Axes or if the xaxis does not
+        If the artist is not contained in an Axes or if the xaxis does not
         have units, *x* itself is returned.
         """
         ax = getattr(self, 'axes', None)
@@ -256,7 +284,7 @@ class Artist:
         """
         Convert *y* using the unit type of the yaxis.
 
-        If the artist is not in contained in an Axes or if the yaxis does not
+        If the artist is not contained in an Axes or if the yaxis does not
         have units, *y* itself is returned.
         """
         ax = getattr(self, 'axes', None)
@@ -301,7 +329,7 @@ class Artist:
         if val and self.stale_callback is not None:
             self.stale_callback(self, val)
 
-    def get_window_extent(self, renderer):
+    def get_window_extent(self, renderer=None):
         """
         Get the artist's bounding box in display space.
 
@@ -321,24 +349,7 @@ class Artist:
         """
         return Bbox([[0, 0], [0, 0]])
 
-    def _get_clipping_extent_bbox(self):
-        """
-        Return a bbox with the extents of the intersection of the clip_path
-        and clip_box for this artist, or None if both of these are
-        None, or ``get_clip_on`` is False.
-        """
-        bbox = None
-        if self.get_clip_on():
-            clip_box = self.get_clip_box()
-            if clip_box is not None:
-                bbox = clip_box
-            clip_path = self.get_clip_path()
-            if clip_path is not None and bbox is not None:
-                clip_path = clip_path.get_fully_transformed_path()
-                bbox = Bbox.intersection(bbox, clip_path.get_extents())
-        return bbox
-
-    def get_tightbbox(self, renderer):
+    def get_tightbbox(self, renderer=None):
         """
         Like `.Artist.get_window_extent`, but includes any clipping.
 
@@ -359,7 +370,7 @@ class Artist:
             if clip_box is not None:
                 bbox = Bbox.intersection(bbox, clip_box)
             clip_path = self.get_clip_path()
-            if clip_path is not None and bbox is not None:
+            if clip_path is not None:
                 clip_path = clip_path.get_fully_transformed_path()
                 bbox = Bbox.intersection(bbox, clip_path.get_extents())
         return bbox
@@ -517,6 +528,7 @@ class Artist:
         --------
         set_picker, get_picker, pickable
         """
+        from .backend_bases import PickEvent  # Circular import.
         # Pick self
         if self.pickable():
             picker = self.get_picker()
@@ -525,20 +537,21 @@ class Artist:
             else:
                 inside, prop = self.contains(mouseevent)
             if inside:
-                self.figure.canvas.pick_event(mouseevent, self, **prop)
+                PickEvent("pick_event", self.figure.canvas,
+                          mouseevent, self, **prop)._process()
 
         # Pick children
         for a in self.get_children():
-            # make sure the event happened in the same axes
+            # make sure the event happened in the same Axes
             ax = getattr(a, 'axes', None)
             if (mouseevent.inaxes is None or ax is None
                     or mouseevent.inaxes == ax):
                 # we need to check if mouseevent.inaxes is None
-                # because some objects associated with an axes (e.g., a
+                # because some objects associated with an Axes (e.g., a
                 # tick label) can be outside the bounding box of the
-                # axes and inaxes will be None
+                # Axes and inaxes will be None
                 # also check that ax is None so that it traverse objects
-                # which do no have an axes property but children might
+                # which do not have an axes property but children might
                 a.pick(mouseevent)
 
     def set_picker(self, picker):
@@ -752,6 +765,11 @@ class Artist:
         Parameters
         ----------
         clipbox : `.Bbox`
+
+            Typically would be created from a `.TransformedBbox`. For
+            instance ``TransformedBbox(Bbox([[0, 0], [1, 1]]), ax.transAxes)``
+            is the default clipping for an artist added to an Axes.
+
         """
         self.clipbox = clipbox
         self.pchanged()
@@ -845,6 +863,30 @@ class Artist:
         """
         return self._in_layout
 
+    def _fully_clipped_to_axes(self):
+        """
+        Return a boolean flag, ``True`` if the artist is clipped to the Axes
+        and can thus be skipped in layout calculations. Requires `get_clip_on`
+        is True, one of `clip_box` or `clip_path` is set, ``clip_box.extents``
+        is equivalent to ``ax.bbox.extents`` (if set), and ``clip_path._patch``
+        is equivalent to ``ax.patch`` (if set).
+        """
+        # Note that ``clip_path.get_fully_transformed_path().get_extents()``
+        # cannot be directly compared to ``axes.bbox.extents`` because the
+        # extents may be undefined (i.e. equivalent to ``Bbox.null()``)
+        # before the associated artist is drawn, and this method is meant
+        # to determine whether ``axes.get_tightbbox()`` may bypass drawing
+        clip_box = self.get_clip_box()
+        clip_path = self.get_clip_path()
+        return (self.axes is not None
+                and self.get_clip_on()
+                and (clip_box is not None or clip_path is not None)
+                and (clip_box is None
+                     or np.all(clip_box.extents == self.axes.bbox.extents))
+                and (clip_path is None
+                     or isinstance(clip_path, TransformedPatchPath)
+                     and clip_path._patch is self.axes.patch))
+
     def get_clip_on(self):
         """Return whether the artist uses clipping."""
         return self._clipon
@@ -871,7 +913,7 @@ class Artist:
         """
         Set whether the artist uses clipping.
 
-        When False artists will be visible outside of the axes which
+        When False, artists will be visible outside the Axes which
         can lead to unexpected results.
 
         Parameters
@@ -914,7 +956,9 @@ class Artist:
         ----------
         rasterized : bool
         """
-        if rasterized and not hasattr(self.draw, "_supports_rasterization"):
+        supports_rasterization = getattr(self.draw,
+                                         "_supports_rasterization", False)
+        if rasterized and not supports_rasterization:
             _api.warn_external(f"Rasterization of '{self}' will be ignored")
 
         self._rasterized = rasterized
@@ -1019,7 +1063,7 @@ class Artist:
 
         If True, the artist is excluded from regular drawing of the figure.
         You have to call `.Figure.draw_artist` / `.Axes.draw_artist`
-        explicitly on the artist. This appoach is used to speed up animations
+        explicitly on the artist. This approach is used to speed up animations
         using blitting.
 
         See also `matplotlib.animation` and
@@ -1046,32 +1090,6 @@ class Artist:
         """
         self._in_layout = in_layout
 
-    def update(self, props):
-        """
-        Update this artist's properties from the dict *props*.
-
-        Parameters
-        ----------
-        props : dict
-        """
-        ret = []
-        with cbook._setattr_cm(self, eventson=False):
-            for k, v in props.items():
-                # Allow attributes we want to be able to update through
-                # art.update, art.set, setp.
-                if k == "axes":
-                    ret.append(setattr(self, k, v))
-                else:
-                    func = getattr(self, f"set_{k}", None)
-                    if not callable(func):
-                        raise AttributeError(f"{type(self).__name__!r} object "
-                                             f"has no property {k!r}")
-                    ret.append(func(v))
-        if ret:
-            self.pchanged()
-            self.stale = True
-        return ret
-
     def get_label(self):
         """Return the label used for this artist in the legend."""
         return self._label
@@ -1159,12 +1177,58 @@ class Artist:
         """Return a dictionary of all the properties of the artist."""
         return ArtistInspector(self).properties()
 
+    def _update_props(self, props, errfmt):
+        """
+        Helper for `.Artist.set` and `.Artist.update`.
+
+        *errfmt* is used to generate error messages for invalid property
+        names; it gets formatted with ``type(self)`` and the property name.
+        """
+        ret = []
+        with cbook._setattr_cm(self, eventson=False):
+            for k, v in props.items():
+                # Allow attributes we want to be able to update through
+                # art.update, art.set, setp.
+                if k == "axes":
+                    ret.append(setattr(self, k, v))
+                else:
+                    func = getattr(self, f"set_{k}", None)
+                    if not callable(func):
+                        raise AttributeError(
+                            errfmt.format(cls=type(self), prop_name=k))
+                    ret.append(func(v))
+        if ret:
+            self.pchanged()
+            self.stale = True
+        return ret
+
+    def update(self, props):
+        """
+        Update this artist's properties from the dict *props*.
+
+        Parameters
+        ----------
+        props : dict
+        """
+        return self._update_props(
+            props, "{cls.__name__!r} object has no property {prop_name!r}")
+
+    def _internal_update(self, kwargs):
+        """
+        Update artist properties without prenormalizing them, but generating
+        errors as if calling `set`.
+
+        The lack of prenormalization is to maintain backcompatibility.
+        """
+        return self._update_props(
+            kwargs, "{cls.__name__}.set() got an unexpected keyword argument "
+            "{prop_name!r}")
+
     def set(self, **kwargs):
         # docstring and signature are auto-generated via
         # Artist._update_set_signature_and_docstring() at the end of the
         # module.
-        kwargs = cbook.normalize_kwargs(kwargs, self)
-        return self.update(kwargs)
+        return self._internal_update(cbook.normalize_kwargs(kwargs, self))
 
     @contextlib.contextmanager
     def _cm_set(self, **kwargs):
@@ -1308,28 +1372,50 @@ class Artist:
                                  if isinstance(item, Number))
             return "[" + data_str + "]"
 
-    @property
-    def mouseover(self):
+    def get_mouseover(self):
         """
-        If this property is set to *True*, the artist will be queried for
-        custom context information when the mouse cursor moves over it.
-
-        See also :meth:`get_cursor_data`, :class:`.ToolCursorPosition` and
-        :class:`.NavigationToolbar2`.
+        Return whether this artist is queried for custom context information
+        when the mouse cursor moves over it.
         """
         return self._mouseover
 
-    @mouseover.setter
-    def mouseover(self, val):
-        val = bool(val)
-        self._mouseover = val
+    def set_mouseover(self, mouseover):
+        """
+        Set whether this artist is queried for custom context information when
+        the mouse cursor moves over it.
+
+        Parameters
+        ----------
+        mouseover : bool
+
+        See Also
+        --------
+        get_cursor_data
+        .ToolCursorPosition
+        .NavigationToolbar2
+        """
+        self._mouseover = bool(mouseover)
         ax = self.axes
         if ax:
-            if val:
+            if self._mouseover:
                 ax._mouseover_set.add(self)
             else:
                 ax._mouseover_set.discard(self)
 
+    mouseover = property(get_mouseover, set_mouseover)  # backcompat.
+
+
+def _get_tightbbox_for_layout_only(obj, *args, **kwargs):
+    """
+    Matplotlib's `.Axes.get_tightbbox` and `.Axis.get_tightbbox` support a
+    *for_layout_only* kwarg; this helper tries to use the kwarg but skips it
+    when encountering third-party subclasses that do not support it.
+    """
+    try:
+        return obj.get_tightbbox(*args, **{**kwargs, "for_layout_only": True})
+    except TypeError:
+        return obj.get_tightbbox(*args, **kwargs)
+
 
 class ArtistInspector:
     """
@@ -1414,7 +1500,7 @@ class ArtistInspector:
         # although barely relevant wrt. matplotlib's total import time.
         param_name = func.__code__.co_varnames[1]
         # We could set the presence * based on whether the parameter is a
-        # varargs (it can't be a varkwargs) but it's not really worth the it.
+        # varargs (it can't be a varkwargs) but it's not really worth it.
         match = re.search(r"(?m)^ *\*?{} : (.+)".format(param_name), docstring)
         if match:
             return match.group(1)
@@ -1445,24 +1531,36 @@ class ArtistInspector:
                 continue
             func = getattr(self.o, name)
             if (not callable(func)
-                    or len(inspect.signature(func).parameters) < 2
+                    or self.number_of_parameters(func) < 2
                     or self.is_alias(func)):
                 continue
             setters.append(name[4:])
         return setters
 
-    def is_alias(self, o):
-        """Return whether method object *o* is an alias for another method."""
-        ds = inspect.getdoc(o)
+    @staticmethod
+    @lru_cache(maxsize=None)
+    def number_of_parameters(func):
+        """Return number of parameters of the callable *func*."""
+        return len(inspect.signature(func).parameters)
+
+    @staticmethod
+    @lru_cache(maxsize=None)
+    def is_alias(method):
+        """
+        Return whether the object *method* is an alias for another method.
+        """
+
+        ds = inspect.getdoc(method)
         if ds is None:
             return False
+
         return ds.startswith('Alias for ')
 
     def aliased_name(self, s):
         """
         Return 'PROPNAME or alias' if *s* has an alias, else return 'PROPNAME'.
 
-        e.g., for the line markerfacecolor property, which has an
+        For example, for the line markerfacecolor property, which has an
         alias, return 'markerfacecolor or mfc' and for the transform
         property, which does not, return 'transform'.
         """
@@ -1490,7 +1588,7 @@ class ArtistInspector:
         Return 'PROPNAME or alias' if *s* has an alias, else return 'PROPNAME',
         formatted for reST.
 
-        e.g., for the line markerfacecolor property, which has an
+        For example, for the line markerfacecolor property, which has an
         alias, return 'markerfacecolor or mfc' and for the transform
         property, which does not, return 'transform'.
         """
