@@ -1,19 +1,8 @@
-"""
-`ToolManager`
-    Class that makes the bridge between user interaction (key press,
-    toolbar clicks, ..) and the actions in response to the user inputs.
-"""
-
-import warnings
-
-import matplotlib.cbook as cbook
-import matplotlib.widgets as widgets
-from matplotlib.rcsetup import validate_stringlist
-import matplotlib.backend_tools as tools
+from matplotlib import _api, backend_tools, cbook, widgets
 
 
-class ToolEvent(object):
-    """Event for tool manipulation (add/remove)"""
+class ToolEvent:
+    """Event for tool manipulation (add/remove)."""
     def __init__(self, name, sender, tool, data=None):
         self.name = name
         self.sender = sender
@@ -22,17 +11,17 @@ class ToolEvent(object):
 
 
 class ToolTriggerEvent(ToolEvent):
-    """Event to inform that a tool has been triggered"""
+    """Event to inform that a tool has been triggered."""
     def __init__(self, name, sender, tool, canvasevent=None, data=None):
-        ToolEvent.__init__(self, name, sender, tool, data)
+        super().__init__(name, sender, tool, data)
         self.canvasevent = canvasevent
 
 
-class ToolManagerMessageEvent(object):
+class ToolManagerMessageEvent:
     """
-    Event carrying messages from toolmanager
+    Event carrying messages from toolmanager.
 
-    Messages usually get displayed to the user by the toolbar
+    Messages usually get displayed to the user by the toolbar.
     """
     def __init__(self, name, sender, message):
         self.name = name
@@ -40,23 +29,21 @@ class ToolManagerMessageEvent(object):
         self.message = message
 
 
-class ToolManager(object):
+class ToolManager:
     """
-    Helper class that groups all the user interactions for a Figure.
+    Manager for actions triggered by user interactions (key press, toolbar
+    clicks, ...) on a Figure.
 
     Attributes
     ----------
-    figure: `Figure`
-    keypresslock: `widgets.LockDraw`
-        `LockDraw` object to know if the `canvas` key_press_event is locked
-    messagelock: `widgets.LockDraw`
-        `LockDraw` object to know if the message is available to write
+    figure : `.Figure`
+    keypresslock : `~matplotlib.widgets.LockDraw`
+        `.LockDraw` object to know if the `canvas` key_press_event is locked.
+    messagelock : `~matplotlib.widgets.LockDraw`
+        `.LockDraw` object to know if the message is available to write.
     """
 
     def __init__(self, figure=None):
-        warnings.warn('Treat the new Tool classes introduced in v1.5 as ' +
-                       'experimental for now, the API will likely change in ' +
-                       'version 2.1 and perhaps the rcParam as well')
 
         self._key_press_handler_id = None
 
@@ -74,14 +61,14 @@ class ToolManager(object):
 
     @property
     def canvas(self):
-        """Canvas managed by FigureManager"""
+        """Canvas managed by FigureManager."""
         if not self._figure:
             return None
         return self._figure.canvas
 
     @property
     def figure(self):
-        """Figure that holds the canvas"""
+        """Figure that holds the canvas."""
         return self._figure
 
     @figure.setter
@@ -95,8 +82,8 @@ class ToolManager(object):
         Parameters
         ----------
         figure : `.Figure`
-        update_tools : bool
-            Force tools to update figure
+        update_tools : bool, default: True
+            Force tools to update figure.
         """
         if self._key_press_handler_id:
             self.canvas.mpl_disconnect(self._key_press_handler_id)
@@ -114,10 +101,8 @@ class ToolManager(object):
 
         Parameters
         ----------
-        s : String
-            Name of the event
-
-            The following events are recognized
+        s : str
+            The name of the event. The following events are recognized:
 
             - 'tool_message_event'
             - 'tool_removed_event'
@@ -125,30 +110,35 @@ class ToolManager(object):
 
             For every tool added a new event is created
 
-            - 'tool_trigger_TOOLNAME`
-              Where TOOLNAME is the id of the tool.
+            - 'tool_trigger_TOOLNAME', where TOOLNAME is the id of the tool.
 
-        func : function
-            Function to be called with signature
-            def func(event)
+        func : callable
+            Callback function for the toolmanager event with signature::
+
+                def func(event: ToolEvent) -> Any
+
+        Returns
+        -------
+        cid
+            The callback id for the connection. This can be used in
+            `.toolmanager_disconnect`.
         """
         return self._callbacks.connect(s, func)
 
     def toolmanager_disconnect(self, cid):
         """
-        Disconnect callback id *cid*
+        Disconnect callback id *cid*.
 
         Example usage::
 
-            cid = toolmanager.toolmanager_connect('tool_trigger_zoom',
-                                                  on_press)
+            cid = toolmanager.toolmanager_connect('tool_trigger_zoom', onpress)
             #...later
             toolmanager.toolmanager_disconnect(cid)
         """
         return self._callbacks.disconnect(cid)
 
     def message_event(self, message, sender=None):
-        """ Emit a `ToolManagerMessageEvent`"""
+        """Emit a `ToolManagerMessageEvent`."""
         if sender is None:
             sender = self
 
@@ -158,22 +148,22 @@ class ToolManager(object):
 
     @property
     def active_toggle(self):
-        """Currently toggled tools"""
-
+        """Currently toggled tools."""
         return self._toggled
 
     def get_tool_keymap(self, name):
         """
-        Get the keymap associated with the specified tool
+        Return the keymap associated with the specified tool.
 
         Parameters
         ----------
-        name : string
-            Name of the Tool
+        name : str
+            Name of the Tool.
 
         Returns
         -------
-        list : list of keys associated with the Tool
+        list of str
+            List of keys associated with the tool.
         """
 
         keys = [k for k, i in self._keys.items() if i == name]
@@ -183,68 +173,71 @@ class ToolManager(object):
         for k in self.get_tool_keymap(name):
             del self._keys[k]
 
-    def update_keymap(self, name, *keys):
+    def update_keymap(self, name, key):
         """
-        Set the keymap to associate with the specified tool
+        Set the keymap to associate with the specified tool.
 
         Parameters
         ----------
-        name : string
-            Name of the Tool
-        keys : keys to associate with the Tool
+        name : str
+            Name of the Tool.
+        key : str or list of str
+            Keys to associate with the tool.
         """
-
         if name not in self._tools:
-            raise KeyError('%s not in Tools' % name)
-
+            raise KeyError(f'{name!r} not in Tools')
         self._remove_keys(name)
-
-        for key in keys:
-            for k in validate_stringlist(key):
-                if k in self._keys:
-                    warnings.warn('Key %s changed from %s to %s' %
-                                  (k, self._keys[k], name))
-                self._keys[k] = name
+        if isinstance(key, str):
+            key = [key]
+        for k in key:
+            if k in self._keys:
+                _api.warn_external(
+                    f'Key {k} changed from {self._keys[k]} to {name}')
+            self._keys[k] = name
 
     def remove_tool(self, name):
         """
-        Remove tool from `ToolManager`
+        Remove tool named *name*.
 
         Parameters
         ----------
-        name : string
-            Name of the Tool
+        name : str
+            Name of the tool.
         """
 
         tool = self.get_tool(name)
-        tool.destroy()
+        destroy = _api.deprecate_method_override(
+            backend_tools.ToolBase.destroy, tool, since="3.6",
+            alternative="tool_removed_event")
+        if destroy is not None:
+            destroy()
 
-        # If is a toggle tool and toggled, untoggle
+        # If it's a toggle tool and toggled, untoggle
         if getattr(tool, 'toggled', False):
             self.trigger_tool(tool, 'toolmanager')
 
         self._remove_keys(name)
 
-        s = 'tool_removed_event'
-        event = ToolEvent(s, self, tool)
-        self._callbacks.process(s, event)
+        event = ToolEvent('tool_removed_event', self, tool)
+        self._callbacks.process(event.name, event)
 
         del self._tools[name]
 
     def add_tool(self, name, tool, *args, **kwargs):
         """
-        Add *tool* to `ToolManager`
+        Add *tool* to `ToolManager`.
 
-        If successful adds a new event `tool_trigger_name` where **name** is
-        the **name** of the tool, this event is fired everytime
-        the tool is triggered.
+        If successful, adds a new event ``tool_trigger_{name}`` where
+        ``{name}`` is the *name* of the tool; the event is fired every time the
+        tool is triggered.
 
         Parameters
         ----------
         name : str
-            Name of the tool, treated as the ID, has to be unique
-        tool : class_like, i.e. str or type
-            Reference to find the class of the Tool to added.
+            Name of the tool, treated as the ID, has to be unique.
+        tool : type
+            Class of the tool to be added.  A subclass will be used
+            instead if one was registered for the current canvas class.
 
         Notes
         -----
@@ -255,23 +248,23 @@ class ToolManager(object):
         matplotlib.backend_tools.ToolBase : The base class for tools.
         """
 
-        tool_cls = self._get_cls_to_instantiate(tool)
+        tool_cls = backend_tools._find_tool_class(type(self.canvas), tool)
         if not tool_cls:
             raise ValueError('Impossible to find class for %s' % str(tool))
 
         if name in self._tools:
-            warnings.warn('A "Tool class" with the same name already exists, '
-                          'not added')
+            _api.warn_external('A "Tool class" with the same name already '
+                               'exists, not added')
             return self._tools[name]
 
         tool_obj = tool_cls(self, name, *args, **kwargs)
         self._tools[name] = tool_obj
 
-        if tool_cls.default_keymap is not None:
-            self.update_keymap(name, tool_cls.default_keymap)
+        if tool_obj.default_keymap is not None:
+            self.update_keymap(name, tool_obj.default_keymap)
 
         # For toggle tools init the radio_group in self._toggled
-        if isinstance(tool_obj, tools.ToolToggleBase):
+        if isinstance(tool_obj, backend_tools.ToolToggleBase):
             # None group is not mutually exclusive, a set is used to keep track
             # of all toggled tools in this group
             if tool_obj.radio_group is None:
@@ -281,31 +274,26 @@ class ToolManager(object):
 
             # If initially toggled
             if tool_obj.toggled:
-                self._handle_toggle(tool_obj, None, None, None)
+                self._handle_toggle(tool_obj, None, None)
         tool_obj.set_figure(self.figure)
 
-        self._tool_added_event(tool_obj)
+        event = ToolEvent('tool_added_event', self, tool_obj)
+        self._callbacks.process(event.name, event)
+
         return tool_obj
 
-    def _tool_added_event(self, tool):
-        s = 'tool_added_event'
-        event = ToolEvent(s, self, tool)
-        self._callbacks.process(s, event)
-
-    def _handle_toggle(self, tool, sender, canvasevent, data):
+    def _handle_toggle(self, tool, canvasevent, data):
         """
-        Toggle tools, need to untoggle prior to using other Toggle tool
-        Called from trigger_tool
+        Toggle tools, need to untoggle prior to using other Toggle tool.
+        Called from trigger_tool.
 
         Parameters
         ----------
-        tool: Tool object
-        sender: object
-            Object that wishes to trigger the tool
+        tool : `.ToolBase`
         canvasevent : Event
-            Original Canvas event or None
-        data : Object
-            Extra data to pass to the tool when triggering
+            Original Canvas event or None.
+        data : object
+            Extra data to pass to the tool when triggering.
         """
 
         radio_group = tool.radio_group
@@ -337,38 +325,20 @@ class ToolManager(object):
         # Keep track of the toggled tool in the radio_group
         self._toggled[radio_group] = toggled
 
-    def _get_cls_to_instantiate(self, callback_class):
-        # Find the class that corresponds to the tool
-        if isinstance(callback_class, str):
-            # FIXME: make more complete searching structure
-            if callback_class in globals():
-                callback_class = globals()[callback_class]
-            else:
-                mod = 'backend_tools'
-                current_module = __import__(mod,
-                                            globals(), locals(), [mod], 1)
-
-                callback_class = getattr(current_module, callback_class, False)
-        if callable(callback_class):
-            return callback_class
-        else:
-            return None
-
-    def trigger_tool(self, name, sender=None, canvasevent=None,
-                     data=None):
+    def trigger_tool(self, name, sender=None, canvasevent=None, data=None):
         """
-        Trigger a tool and emit the tool_trigger_[name] event
+        Trigger a tool and emit the ``tool_trigger_{name}`` event.
 
         Parameters
         ----------
-        name : string
-            Name of the tool
-        sender: object
-            Object that wishes to trigger the tool
+        name : str
+            Name of the tool.
+        sender : object
+            Object that wishes to trigger the tool.
         canvasevent : Event
-            Original Canvas event or None
-        data : Object
-            Extra data to pass to the tool when triggering
+            Original Canvas event or None.
+        data : object
+            Extra data to pass to the tool when triggering.
         """
         tool = self.get_tool(name)
         if tool is None:
@@ -377,27 +347,15 @@ class ToolManager(object):
         if sender is None:
             sender = self
 
-        self._trigger_tool(name, sender, canvasevent, data)
+        if isinstance(tool, backend_tools.ToolToggleBase):
+            self._handle_toggle(tool, canvasevent, data)
+
+        tool.trigger(sender, canvasevent, data)  # Actually trigger Tool.
 
         s = 'tool_trigger_%s' % name
         event = ToolTriggerEvent(s, sender, tool, canvasevent, data)
         self._callbacks.process(s, event)
 
-    def _trigger_tool(self, name, sender=None, canvasevent=None, data=None):
-        """
-        Trigger on a tool
-
-        Method to actually trigger the tool
-        """
-        tool = self.get_tool(name)
-
-        if isinstance(tool, tools.ToolToggleBase):
-            self._handle_toggle(tool, sender, canvasevent, data)
-
-        # Important!!!
-        # This is where the Tool object gets triggered
-        tool.trigger(sender, canvasevent, data)
-
     def _key_press(self, event):
         if event.key is None or self.keypresslock.locked():
             return
@@ -409,25 +367,34 @@ class ToolManager(object):
 
     @property
     def tools(self):
-        """Return the tools controlled by `ToolManager`"""
-
+        """A dict mapping tool name -> controlled tool."""
         return self._tools
 
     def get_tool(self, name, warn=True):
         """
-        Return the tool object, also accepts the actual tool for convenience
+        Return the tool object with the given name.
+
+        For convenience, this passes tool objects through.
 
         Parameters
         ----------
-        name : str, ToolBase
-            Name of the tool, or the tool itself
-        warn : bool, optional
-            If this method should give warnings.
+        name : str or `.ToolBase`
+            Name of the tool, or the tool itself.
+        warn : bool, default: True
+            Whether a warning should be emitted it no tool with the given name
+            exists.
+
+        Returns
+        -------
+        `.ToolBase` or None
+            The tool or None if no tool with the given name exists.
         """
-        if isinstance(name, tools.ToolBase) and name.name in self._tools:
+        if (isinstance(name, backend_tools.ToolBase)
+                and name.name in self._tools):
             return name
         if name not in self._tools:
             if warn:
-                warnings.warn("ToolManager does not control tool %s" % name)
+                _api.warn_external(
+                    f"ToolManager does not control tool {name!r}")
             return None
         return self._tools[name]
