@@ -14,14 +14,10 @@ import matplotlib.backends.qt_editor.figureoptions as figureoptions
 from . import qt_compat
 from .qt_compat import (
     QtCore, QtGui, QtWidgets, __version__, QT_API,
-    _enum, _to_int,
-    _devicePixelRatioF, _isdeleted, _setDevicePixelRatio,
-    _maybe_allow_interrupt
+    _enum, _to_int, _isdeleted, _maybe_allow_interrupt
 )
 
 
-backend_version = __version__
-
 # SPECIAL_KEYS are Qt::Key that do *not* return their Unicode name
 # instead they have manually specified names.
 SPECIAL_KEYS = {
@@ -105,8 +101,8 @@ class __getattr__:
 def _create_qApp():
     app = QtWidgets.QApplication.instance()
 
-    # Create a new QApplication and configure if if non exists yet, as only one
-    # QApplication can exist at a time.
+    # Create a new QApplication and configure it if none exists yet, as only
+    # one QApplication can exist at a time.
     if app is None:
         # display_is_valid returns False only if on Linux and neither X11
         # nor Wayland display can be opened.
@@ -222,7 +218,8 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         self.setPalette(palette)
 
     def _update_pixel_ratio(self):
-        if self._set_device_pixel_ratio(_devicePixelRatioF(self)):
+        if self._set_device_pixel_ratio(
+                self.devicePixelRatioF() or 1):  # rarely, devicePixelRatioF=0
             # The easiest way to resize the canvas is to emit a resizeEvent
             # since we implement all the logic for resizing the canvas for
             # that event.
@@ -270,14 +267,19 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         return x * self.device_pixel_ratio, y * self.device_pixel_ratio
 
     def enterEvent(self, event):
+        # Force querying of the modifiers, as the cached modifier state can
+        # have been invalidated while the window was out of focus.
+        mods = QtWidgets.QApplication.instance().queryKeyboardModifiers()
         LocationEvent("figure_enter_event", self,
                       *self.mouseEventCoords(event),
+                      modifiers=self._mpl_modifiers(mods),
                       guiEvent=event)._process()
 
     def leaveEvent(self, event):
         QtWidgets.QApplication.restoreOverrideCursor()
         LocationEvent("figure_leave_event", self,
                       *self.mouseEventCoords(),
+                      modifiers=self._mpl_modifiers(),
                       guiEvent=event)._process()
 
     def mousePressEvent(self, event):
@@ -285,6 +287,7 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         if button is not None:
             MouseEvent("button_press_event", self,
                        *self.mouseEventCoords(event), button,
+                       modifiers=self._mpl_modifiers(),
                        guiEvent=event)._process()
 
     def mouseDoubleClickEvent(self, event):
@@ -292,11 +295,13 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         if button is not None:
             MouseEvent("button_press_event", self,
                        *self.mouseEventCoords(event), button, dblclick=True,
+                       modifiers=self._mpl_modifiers(),
                        guiEvent=event)._process()
 
     def mouseMoveEvent(self, event):
         MouseEvent("motion_notify_event", self,
                    *self.mouseEventCoords(event),
+                   modifiers=self._mpl_modifiers(),
                    guiEvent=event)._process()
 
     def mouseReleaseEvent(self, event):
@@ -304,6 +309,7 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         if button is not None:
             MouseEvent("button_release_event", self,
                        *self.mouseEventCoords(event), button,
+                       modifiers=self._mpl_modifiers(),
                        guiEvent=event)._process()
 
     def wheelEvent(self, event):
@@ -317,6 +323,7 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         if steps:
             MouseEvent("scroll_event", self,
                        *self.mouseEventCoords(event), step=steps,
+                       modifiers=self._mpl_modifiers(),
                        guiEvent=event)._process()
 
     def keyPressEvent(self, event):
@@ -359,18 +366,23 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
     def minumumSizeHint(self):
         return QtCore.QSize(10, 10)
 
-    def _get_key(self, event):
-        event_key = event.key()
-        event_mods = _to_int(event.modifiers())  # actually a bitmask
-
+    @staticmethod
+    def _mpl_modifiers(modifiers=None, *, exclude=None):
+        if modifiers is None:
+            modifiers = QtWidgets.QApplication.instance().keyboardModifiers()
+        modifiers = _to_int(modifiers)
         # get names of the pressed modifier keys
         # 'control' is named 'control' when a standalone key, but 'ctrl' when a
         # modifier
-        # bit twiddling to pick out modifier keys from event_mods bitmask,
-        # if event_key is a MODIFIER, it should not be duplicated in mods
-        mods = [SPECIAL_KEYS[key].replace('control', 'ctrl')
-                for mod, key in _MODIFIER_KEYS
-                if event_key != key and event_mods & mod]
+        # bit twiddling to pick out modifier keys from modifiers bitmask,
+        # if exclude is a MODIFIER, it should not be duplicated in mods
+        return [SPECIAL_KEYS[key].replace('control', 'ctrl')
+                for mask, key in _MODIFIER_KEYS
+                if exclude != key and modifiers & mask]
+
+    def _get_key(self, event):
+        event_key = event.key()
+        mods = self._mpl_modifiers(exclude=event_key)
         try:
             # for certain keys (enter, left, backspace, etc) use a word for the
             # key, rather than Unicode
@@ -378,7 +390,7 @@ class FigureCanvasQT(FigureCanvasBase, QtWidgets.QWidget):
         except KeyError:
             # Unicode defines code points up to 0x10ffff (sys.maxunicode)
             # QT will use Key_Codes larger than that for keyboard keys that are
-            # are not Unicode characters (like multimedia keys)
+            # not Unicode characters (like multimedia keys)
             # skip these
             # if you really want them, you should add them to SPECIAL_KEYS
             if event_key > sys.maxunicode:
@@ -585,6 +597,13 @@ class FigureManagerQT(FigureManagerBase):
         self.canvas.resize(width, height)
         self.window.resize(width + extra_width, height + extra_height)
 
+    @classmethod
+    def start_main_loop(cls):
+        qapp = QtWidgets.QApplication.instance()
+        if qapp:
+            with _maybe_allow_interrupt(qapp):
+                qt_compat._exec(qapp)
+
     def show(self):
         self.window.show()
         if mpl.rcParams['figure.raise_window']:
@@ -672,7 +691,8 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         filename = str(path_large if path_large.exists() else path_regular)
 
         pm = QtGui.QPixmap(filename)
-        _setDevicePixelRatio(pm, _devicePixelRatioF(self))
+        pm.setDevicePixelRatio(
+            self.devicePixelRatioF() or 1)  # rarely, devicePixelRatioF=0
         if self.palette().color(self.backgroundRole()).value() < 128:
             icon_color = self.palette().color(self.foregroundRole())
             mask = pm.createMaskFromColor(
@@ -976,13 +996,6 @@ class SaveFigureQt(backend_tools.SaveFigureBase):
             self._make_classic_style_pseudo_toolbar())
 
 
-@_api.deprecated("3.5", alternative="ToolSetCursor")
-class SetCursorQt(backend_tools.SetCursorBase):
-    def set_cursor(self, cursor):
-        NavigationToolbar2QT.set_cursor(
-            self._make_classic_style_pseudo_toolbar(), cursor)
-
-
 @backend_tools._register_tool_class(FigureCanvasQT)
 class RubberbandQt(backend_tools.RubberbandBase):
     def draw_rubberband(self, x0, y0, x1, y1):
@@ -1013,11 +1026,7 @@ FigureManagerQT._toolmanager_toolbar_class = ToolbarQt
 
 @_Backend.export
 class _BackendQT(_Backend):
+    backend_version = __version__
     FigureCanvas = FigureCanvasQT
     FigureManager = FigureManagerQT
-
-    @staticmethod
-    def mainloop():
-        qapp = QtWidgets.QApplication.instance()
-        with _maybe_allow_interrupt(qapp):
-            qt_compat._exec(qapp)
+    mainloop = FigureManagerQT.start_main_loop
