@@ -51,7 +51,7 @@ class Test_delete_masked_points:
 
 
 class Test_boxplot_stats:
-    def setup(self):
+    def setup_method(self):
         np.random.seed(937)
         self.nrows = 37
         self.ncols = 4
@@ -177,7 +177,7 @@ class Test_boxplot_stats:
 
 
 class Test_callback_registry:
-    def setup(self):
+    def setup_method(self):
         self.signal = 'test'
         self.callbacks = cbook.CallbackRegistry()
 
@@ -895,3 +895,19 @@ def test_safe_first_element_with_none():
     datetime_lst[0] = None
     actual = cbook._safe_first_finite(datetime_lst)
     assert actual is not None and actual == datetime_lst[1]
+
+
+@pytest.mark.parametrize('fmt, value, result', [
+    ('%.2f m', 0.2, '0.20 m'),
+    ('{:.2f} m', 0.2, '0.20 m'),
+    ('{} m', 0.2, '0.2 m'),
+    ('const', 0.2, 'const'),
+    ('%d or {}', 0.2, '0 or {}'),
+    ('{{{:,.0f}}}', 2e5, '{200,000}'),
+    ('{:.2%}', 2/3, '66.67%'),
+    ('$%g', 2.54, '$2.54'),
+])
+def test_auto_format_str(fmt, value, result):
+    """Apply *value* to the format string *fmt*."""
+    assert cbook._auto_format_str(fmt, value) == result
+    assert cbook._auto_format_str(fmt, np.float64(value)) == result
