@@ -2,12 +2,14 @@ import datetime
 import platform
 import re
 
+import contourpy
 import numpy as np
-from numpy.testing import assert_array_almost_equal
+from numpy.testing import (
+    assert_array_almost_equal, assert_array_almost_equal_nulp)
 import matplotlib as mpl
 from matplotlib.testing.decorators import image_comparison
-from matplotlib import pyplot as plt, rc_context
-from matplotlib.colors import LogNorm
+from matplotlib import pyplot as plt, rc_context, ticker
+from matplotlib.colors import LogNorm, same_color
 import pytest
 
 
@@ -60,15 +62,16 @@ def test_contour_shape_error(args, message):
         ax.contour(*args)
 
 
-def test_contour_empty_levels():
-
-    x = np.arange(9)
-    z = np.random.random((9, 9))
-
+def test_contour_no_valid_levels():
     fig, ax = plt.subplots()
-    with pytest.warns(UserWarning) as record:
-        ax.contour(x, x, z, levels=[])
-    assert len(record) == 1
+    # no warning for empty levels.
+    ax.contour(np.random.rand(9, 9), levels=[])
+    # no warning if levels is given and is not within the range of z.
+    cs = ax.contour(np.arange(81).reshape((9, 9)), levels=[100])
+    # ... and if fmt is given.
+    ax.clabel(cs, fmt={100: '%1.2f'})
+    # no warning if z is uniform.
+    ax.contour(np.ones((9, 9)))
 
 
 def test_contour_Nlevels():
@@ -82,33 +85,6 @@ def test_contour_Nlevels():
     assert (cs1.levels == cs2.levels).all()
 
 
-def test_contour_badlevel_fmt():
-    # Test edge case from https://github.com/matplotlib/matplotlib/issues/9742
-    # User supplied fmt for each level as a dictionary, but Matplotlib changed
-    # the level to the minimum data value because no contours possible.
-    # This was fixed in https://github.com/matplotlib/matplotlib/pull/9743
-    x = np.arange(9)
-    z = np.zeros((9, 9))
-
-    fig, ax = plt.subplots()
-    fmt = {1.: '%1.2f'}
-    with pytest.warns(UserWarning) as record:
-        cs = ax.contour(x, x, z, levels=[1.])
-        ax.clabel(cs, fmt=fmt)
-    assert len(record) == 1
-
-
-def test_contour_uniform_z():
-
-    x = np.arange(9)
-    z = np.ones((9, 9))
-
-    fig, ax = plt.subplots()
-    with pytest.warns(UserWarning) as record:
-        ax.contour(x, x, z)
-    assert len(record) == 1
-
-
 @image_comparison(['contour_manual_labels'], remove_text=True, style='mpl20')
 def test_contour_manual_labels():
     x, y = np.meshgrid(np.arange(0, 10), np.arange(0, 10))
@@ -153,8 +129,29 @@ def test_given_colors_levels_and_extends():
         plt.colorbar(c, ax=ax)
 
 
-@image_comparison(['contour_datetime_axis.png'],
-                  remove_text=False, style='mpl20')
+@image_comparison(['contour_log_locator.svg'], style='mpl20',
+                  remove_text=False)
+def test_log_locator_levels():
+
+    fig, ax = plt.subplots()
+
+    N = 100
+    x = np.linspace(-3.0, 3.0, N)
+    y = np.linspace(-2.0, 2.0, N)
+
+    X, Y = np.meshgrid(x, y)
+
+    Z1 = np.exp(-X**2 - Y**2)
+    Z2 = np.exp(-(X * 10)**2 - (Y * 10)**2)
+    data = Z1 + 50 * Z2
+
+    c = ax.contourf(data, locator=ticker.LogLocator())
+    assert_array_almost_equal(c.levels, np.power(10.0, np.arange(-6, 3)))
+    cb = fig.colorbar(c, ax=ax)
+    assert_array_almost_equal(cb.ax.get_yticks(), c.levels)
+
+
+@image_comparison(['contour_datetime_axis.png'], style='mpl20')
 def test_contour_datetime_axis():
     fig = plt.figure()
     fig.subplots_adjust(hspace=0.4, top=0.98, bottom=.15)
@@ -244,33 +241,6 @@ def test_contourf_symmetric_locator():
     assert_array_almost_equal(cs.levels, np.linspace(-12, 12, 5))
 
 
-@pytest.mark.parametrize("args, cls, message", [
-    ((), TypeError,
-     'function takes exactly 6 arguments (0 given)'),
-    ((1, 2, 3, 4, 5, 6), ValueError,
-     'Expected 2-dimensional array, got 0'),
-    (([[0]], [[0]], [[]], None, True, 0), ValueError,
-     'x, y and z must all be 2D arrays with the same dimensions'),
-    (([[0]], [[0]], [[0]], None, True, 0), ValueError,
-     'x, y and z must all be at least 2x2 arrays'),
-    ((*[np.arange(4).reshape((2, 2))] * 3, [[0]], True, 0), ValueError,
-     'If mask is set it must be a 2D array with the same dimensions as x.'),
-])
-def test_internal_cpp_api(args, cls, message):  # Github issue 8197.
-    from matplotlib import _contour  # noqa: ensure lazy-loaded module *is* loaded.
-    with pytest.raises(cls, match=re.escape(message)):
-        mpl._contour.QuadContourGenerator(*args)
-
-
-def test_internal_cpp_api_2():
-    from matplotlib import _contour  # noqa: ensure lazy-loaded module *is* loaded.
-    arr = [[0, 1], [2, 3]]
-    qcg = mpl._contour.QuadContourGenerator(arr, arr, arr, None, True, 0)
-    with pytest.raises(
-            ValueError, match=r'filled contour levels must be increasing'):
-        qcg.create_filled_contour(1, 0)
-
-
 def test_circular_contour_warning():
     # Check that almost circular contours don't throw a warning
     x, y = np.meshgrid(np.linspace(-2, 2, 4), np.linspace(-2, 2, 4))
@@ -340,12 +310,14 @@ def test_contourf_log_extension():
     cb = plt.colorbar(c1, ax=ax1)
     assert cb.ax.get_ylim() == (1e-8, 1e10)
     cb = plt.colorbar(c2, ax=ax2)
-    assert cb.ax.get_ylim() == (1e-4, 1e6)
+    assert_array_almost_equal_nulp(cb.ax.get_ylim(), np.array((1e-4, 1e6)))
     cb = plt.colorbar(c3, ax=ax3)
 
 
-@image_comparison(['contour_addlines.png'],
-                  remove_text=True, style='mpl20', tol=0.03)
+@image_comparison(
+    ['contour_addlines.png'], remove_text=True, style='mpl20',
+    tol=0.15 if platform.machine() in ('aarch64', 'ppc64le', 's390x')
+        else 0.03)
 # tolerance is because image changed minutely when tick finding on
 # colorbars was cleaned up...
 def test_contour_addlines():
@@ -520,3 +492,219 @@ def test_contour_autolabel_beyond_powerlimits():
     ax.clabel(cs)
     # Currently, the exponent is missing, but that may be fixed in the future.
     assert {text.get_text() for text in ax.texts} == {"0.25", "1.00", "4.00"}
+
+
+def test_contourf_legend_elements():
+    from matplotlib.patches import Rectangle
+    x = np.arange(1, 10)
+    y = x.reshape(-1, 1)
+    h = x * y
+
+    cs = plt.contourf(h, levels=[10, 30, 50],
+                      colors=['#FFFF00', '#FF00FF', '#00FFFF'],
+                      extend='both')
+    cs.cmap.set_over('red')
+    cs.cmap.set_under('blue')
+    cs.changed()
+    artists, labels = cs.legend_elements()
+    assert labels == ['$x \\leq -1e+250s$',
+                      '$10.0 < x \\leq 30.0$',
+                      '$30.0 < x \\leq 50.0$',
+                      '$x > 1e+250s$']
+    expected_colors = ('blue', '#FFFF00', '#FF00FF', 'red')
+    assert all(isinstance(a, Rectangle) for a in artists)
+    assert all(same_color(a.get_facecolor(), c)
+               for a, c in zip(artists, expected_colors))
+
+
+def test_contour_legend_elements():
+    from matplotlib.collections import LineCollection
+    x = np.arange(1, 10)
+    y = x.reshape(-1, 1)
+    h = x * y
+
+    colors = ['blue', '#00FF00', 'red']
+    cs = plt.contour(h, levels=[10, 30, 50],
+                     colors=colors,
+                     extend='both')
+    artists, labels = cs.legend_elements()
+    assert labels == ['$x = 10.0$', '$x = 30.0$', '$x = 50.0$']
+    assert all(isinstance(a, LineCollection) for a in artists)
+    assert all(same_color(a.get_color(), c)
+               for a, c in zip(artists, colors))
+
+
+@pytest.mark.parametrize(
+    "algorithm, klass",
+    [('mpl2005', contourpy.Mpl2005ContourGenerator),
+     ('mpl2014', contourpy.Mpl2014ContourGenerator),
+     ('serial', contourpy.SerialContourGenerator),
+     ('threaded', contourpy.ThreadedContourGenerator),
+     ('invalid', None)])
+def test_algorithm_name(algorithm, klass):
+    z = np.array([[1.0, 2.0], [3.0, 4.0]])
+    if klass is not None:
+        cs = plt.contourf(z, algorithm=algorithm)
+        assert isinstance(cs._contour_generator, klass)
+    else:
+        with pytest.raises(ValueError):
+            plt.contourf(z, algorithm=algorithm)
+
+
+@pytest.mark.parametrize(
+    "algorithm", ['mpl2005', 'mpl2014', 'serial', 'threaded'])
+def test_algorithm_supports_corner_mask(algorithm):
+    z = np.array([[1.0, 2.0], [3.0, 4.0]])
+
+    # All algorithms support corner_mask=False
+    plt.contourf(z, algorithm=algorithm, corner_mask=False)
+
+    # Only some algorithms support corner_mask=True
+    if algorithm != 'mpl2005':
+        plt.contourf(z, algorithm=algorithm, corner_mask=True)
+    else:
+        with pytest.raises(ValueError):
+            plt.contourf(z, algorithm=algorithm, corner_mask=True)
+
+
+@image_comparison(baseline_images=['contour_all_algorithms'],
+                  extensions=['png'], remove_text=True)
+def test_all_algorithms():
+    algorithms = ['mpl2005', 'mpl2014', 'serial', 'threaded']
+
+    rng = np.random.default_rng(2981)
+    x, y = np.meshgrid(np.linspace(0.0, 1.0, 10), np.linspace(0.0, 1.0, 6))
+    z = np.sin(15*x)*np.cos(10*y) + rng.normal(scale=0.5, size=(6, 10))
+    mask = np.zeros_like(z, dtype=bool)
+    mask[3, 7] = True
+    z = np.ma.array(z, mask=mask)
+
+    _, axs = plt.subplots(2, 2)
+    for ax, algorithm in zip(axs.ravel(), algorithms):
+        ax.contourf(x, y, z, algorithm=algorithm)
+        ax.contour(x, y, z, algorithm=algorithm, colors='k')
+        ax.set_title(algorithm)
+
+
+def test_subfigure_clabel():
+    # Smoke test for gh#23173
+    delta = 0.025
+    x = np.arange(-3.0, 3.0, delta)
+    y = np.arange(-2.0, 2.0, delta)
+    X, Y = np.meshgrid(x, y)
+    Z1 = np.exp(-(X**2) - Y**2)
+    Z2 = np.exp(-((X - 1) ** 2) - (Y - 1) ** 2)
+    Z = (Z1 - Z2) * 2
+
+    fig = plt.figure()
+    figs = fig.subfigures(nrows=1, ncols=2)
+
+    for f in figs:
+        ax = f.subplots()
+        CS = ax.contour(X, Y, Z)
+        ax.clabel(CS, inline=True, fontsize=10)
+        ax.set_title("Simplest default with labels")
+
+
+@pytest.mark.parametrize(
+    "style", ['solid', 'dashed', 'dashdot', 'dotted'])
+def test_linestyles(style):
+    delta = 0.025
+    x = np.arange(-3.0, 3.0, delta)
+    y = np.arange(-2.0, 2.0, delta)
+    X, Y = np.meshgrid(x, y)
+    Z1 = np.exp(-X**2 - Y**2)
+    Z2 = np.exp(-(X - 1)**2 - (Y - 1)**2)
+    Z = (Z1 - Z2) * 2
+
+    # Positive contour defaults to solid
+    fig1, ax1 = plt.subplots()
+    CS1 = ax1.contour(X, Y, Z, 6, colors='k')
+    ax1.clabel(CS1, fontsize=9, inline=True)
+    ax1.set_title('Single color - positive contours solid (default)')
+    assert CS1.linestyles is None  # default
+
+    # Change linestyles using linestyles kwarg
+    fig2, ax2 = plt.subplots()
+    CS2 = ax2.contour(X, Y, Z, 6, colors='k', linestyles=style)
+    ax2.clabel(CS2, fontsize=9, inline=True)
+    ax2.set_title(f'Single color - positive contours {style}')
+    assert CS2.linestyles == style
+
+    # Ensure linestyles do not change when negative_linestyles is defined
+    fig3, ax3 = plt.subplots()
+    CS3 = ax3.contour(X, Y, Z, 6, colors='k', linestyles=style,
+                      negative_linestyles='dashdot')
+    ax3.clabel(CS3, fontsize=9, inline=True)
+    ax3.set_title(f'Single color - positive contours {style}')
+    assert CS3.linestyles == style
+
+
+@pytest.mark.parametrize(
+    "style", ['solid', 'dashed', 'dashdot', 'dotted'])
+def test_negative_linestyles(style):
+    delta = 0.025
+    x = np.arange(-3.0, 3.0, delta)
+    y = np.arange(-2.0, 2.0, delta)
+    X, Y = np.meshgrid(x, y)
+    Z1 = np.exp(-X**2 - Y**2)
+    Z2 = np.exp(-(X - 1)**2 - (Y - 1)**2)
+    Z = (Z1 - Z2) * 2
+
+    # Negative contour defaults to dashed
+    fig1, ax1 = plt.subplots()
+    CS1 = ax1.contour(X, Y, Z, 6, colors='k')
+    ax1.clabel(CS1, fontsize=9, inline=True)
+    ax1.set_title('Single color - negative contours dashed (default)')
+    assert CS1.negative_linestyles == 'dashed'  # default
+
+    # Change negative_linestyles using rcParams
+    plt.rcParams['contour.negative_linestyle'] = style
+    fig2, ax2 = plt.subplots()
+    CS2 = ax2.contour(X, Y, Z, 6, colors='k')
+    ax2.clabel(CS2, fontsize=9, inline=True)
+    ax2.set_title(f'Single color - negative contours {style}'
+                   '(using rcParams)')
+    assert CS2.negative_linestyles == style
+
+    # Change negative_linestyles using negative_linestyles kwarg
+    fig3, ax3 = plt.subplots()
+    CS3 = ax3.contour(X, Y, Z, 6, colors='k', negative_linestyles=style)
+    ax3.clabel(CS3, fontsize=9, inline=True)
+    ax3.set_title(f'Single color - negative contours {style}')
+    assert CS3.negative_linestyles == style
+
+    # Ensure negative_linestyles do not change when linestyles is defined
+    fig4, ax4 = plt.subplots()
+    CS4 = ax4.contour(X, Y, Z, 6, colors='k', linestyles='dashdot',
+                      negative_linestyles=style)
+    ax4.clabel(CS4, fontsize=9, inline=True)
+    ax4.set_title(f'Single color - negative contours {style}')
+    assert CS4.negative_linestyles == style
+
+
+def test_contour_remove():
+    ax = plt.figure().add_subplot()
+    orig_children = ax.get_children()
+    cs = ax.contour(np.arange(16).reshape((4, 4)))
+    cs.clabel()
+    assert ax.get_children() != orig_children
+    cs.remove()
+    assert ax.get_children() == orig_children
+
+
+def test_bool_autolevel():
+    x, y = np.random.rand(2, 9)
+    z = (np.arange(9) % 2).reshape((3, 3)).astype(bool)
+    m = [[False, False, False], [False, True, False], [False, False, False]]
+    assert plt.contour(z.tolist()).levels.tolist() == [.5]
+    assert plt.contour(z).levels.tolist() == [.5]
+    assert plt.contour(np.ma.array(z, mask=m)).levels.tolist() == [.5]
+    assert plt.contourf(z.tolist()).levels.tolist() == [0, .5, 1]
+    assert plt.contourf(z).levels.tolist() == [0, .5, 1]
+    assert plt.contourf(np.ma.array(z, mask=m)).levels.tolist() == [0, .5, 1]
+    z = z.ravel()
+    assert plt.tricontour(x, y, z.tolist()).levels.tolist() == [.5]
+    assert plt.tricontour(x, y, z).levels.tolist() == [.5]
+    assert plt.tricontourf(x, y, z.tolist()).levels.tolist() == [0, .5, 1]
+    assert plt.tricontourf(x, y, z).levels.tolist() == [0, .5, 1]
