@@ -32,7 +32,7 @@ const char *Py_point_in_path__doc__ =
     "point_in_path(x, y, radius, path, trans)\n"
     "--\n\n";
 
-static PyObject *Py_point_in_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_point_in_path(PyObject *self, PyObject *args)
 {
     double x, y, r;
     py::PathIterator path;
@@ -64,7 +64,7 @@ const char *Py_points_in_path__doc__ =
     "points_in_path(points, radius, path, trans)\n"
     "--\n\n";
 
-static PyObject *Py_points_in_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_points_in_path(PyObject *self, PyObject *args)
 {
     numpy::array_view<const double, 2> points;
     double r;
@@ -95,7 +95,7 @@ const char *Py_point_on_path__doc__ =
     "point_on_path(x, y, radius, path, trans)\n"
     "--\n\n";
 
-static PyObject *Py_point_on_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_point_on_path(PyObject *self, PyObject *args)
 {
     double x, y, r;
     py::PathIterator path;
@@ -127,7 +127,7 @@ const char *Py_points_on_path__doc__ =
     "points_on_path(points, radius, path, trans)\n"
     "--\n\n";
 
-static PyObject *Py_points_on_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_points_on_path(PyObject *self, PyObject *args)
 {
     numpy::array_view<const double, 2> points;
     double r;
@@ -158,7 +158,7 @@ const char *Py_get_path_extents__doc__ =
     "get_path_extents(path, trans)\n"
     "--\n\n";
 
-static PyObject *Py_get_path_extents(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_get_path_extents(PyObject *self, PyObject *args)
 {
     py::PathIterator path;
     agg::trans_affine trans;
@@ -187,7 +187,7 @@ const char *Py_update_path_extents__doc__ =
     "update_path_extents(path, trans, rect, minpos, ignore)\n"
     "--\n\n";
 
-static PyObject *Py_update_path_extents(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_update_path_extents(PyObject *self, PyObject *args)
 {
     py::PathIterator path;
     agg::trans_affine trans;
@@ -266,20 +266,21 @@ const char *Py_get_path_collection_extents__doc__ =
     "master_transform, paths, transforms, offsets, offset_transform)\n"
     "--\n\n";
 
-static PyObject *Py_get_path_collection_extents(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_get_path_collection_extents(PyObject *self, PyObject *args)
 {
     agg::trans_affine master_transform;
-    PyObject *pathsobj;
+    py::PathGenerator paths;
     numpy::array_view<const double, 3> transforms;
     numpy::array_view<const double, 2> offsets;
     agg::trans_affine offset_trans;
     extent_limits e;
 
     if (!PyArg_ParseTuple(args,
-                          "O&OO&O&O&:get_path_collection_extents",
+                          "O&O&O&O&O&:get_path_collection_extents",
                           &convert_trans_affine,
                           &master_transform,
-                          &pathsobj,
+                          &convert_pathgen,
+                          &paths,
                           &convert_transforms,
                           &transforms,
                           &convert_points,
@@ -289,18 +290,9 @@ static PyObject *Py_get_path_collection_extents(PyObject *self, PyObject *args,
         return NULL;
     }
 
-    try
-    {
-        py::PathGenerator paths(pathsobj);
-
-        CALL_CPP("get_path_collection_extents",
-                 (get_path_collection_extents(
-                     master_transform, paths, transforms, offsets, offset_trans, e)));
-    }
-    catch (const py::exception &)
-    {
-        return NULL;
-    }
+    CALL_CPP("get_path_collection_extents",
+             (get_path_collection_extents(
+                 master_transform, paths, transforms, offsets, offset_trans, e)));
 
     npy_intp dims[] = { 2, 2 };
     numpy::array_view<double, 2> extents(dims);
@@ -320,29 +312,29 @@ static PyObject *Py_get_path_collection_extents(PyObject *self, PyObject *args,
 const char *Py_point_in_path_collection__doc__ =
     "point_in_path_collection("
     "x, y, radius, master_transform, paths, transforms, offsets, "
-    "offset_trans, filled, offset_position)\n"
+    "offset_trans, filled)\n"
     "--\n\n";
 
-static PyObject *Py_point_in_path_collection(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_point_in_path_collection(PyObject *self, PyObject *args)
 {
     double x, y, radius;
     agg::trans_affine master_transform;
-    PyObject *pathsobj;
+    py::PathGenerator paths;
     numpy::array_view<const double, 3> transforms;
     numpy::array_view<const double, 2> offsets;
     agg::trans_affine offset_trans;
     bool filled;
-    e_offset_position offset_position;
     std::vector<int> result;
 
     if (!PyArg_ParseTuple(args,
-                          "dddO&OO&O&O&O&O&:point_in_path_collection",
+                          "dddO&O&O&O&O&O&:point_in_path_collection",
                           &x,
                           &y,
                           &radius,
                           &convert_trans_affine,
                           &master_transform,
-                          &pathsobj,
+                          &convert_pathgen,
+                          &paths,
                           &convert_transforms,
                           &transforms,
                           &convert_points,
@@ -350,33 +342,21 @@ static PyObject *Py_point_in_path_collection(PyObject *self, PyObject *args, PyO
                           &convert_trans_affine,
                           &offset_trans,
                           &convert_bool,
-                          &filled,
-                          &convert_offset_position,
-                          &offset_position)) {
+                          &filled)) {
         return NULL;
     }
 
-    try
-    {
-        py::PathGenerator paths(pathsobj);
-
-        CALL_CPP("point_in_path_collection",
-                 (point_in_path_collection(x,
-                                           y,
-                                           radius,
-                                           master_transform,
-                                           paths,
-                                           transforms,
-                                           offsets,
-                                           offset_trans,
-                                           filled,
-                                           offset_position,
-                                           result)));
-    }
-    catch (const py::exception &)
-    {
-        return NULL;
-    }
+    CALL_CPP("point_in_path_collection",
+             (point_in_path_collection(x,
+                                       y,
+                                       radius,
+                                       master_transform,
+                                       paths,
+                                       transforms,
+                                       offsets,
+                                       offset_trans,
+                                       filled,
+                                       result)));
 
     npy_intp dims[] = {(npy_intp)result.size() };
     numpy::array_view<int, 1> pyresult(dims);
@@ -390,7 +370,7 @@ const char *Py_path_in_path__doc__ =
     "path_in_path(path_a, trans_a, path_b, trans_b)\n"
     "--\n\n";
 
-static PyObject *Py_path_in_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_path_in_path(PyObject *self, PyObject *args)
 {
     py::PathIterator a;
     agg::trans_affine atrans;
@@ -424,7 +404,7 @@ const char *Py_clip_path_to_rect__doc__ =
     "clip_path_to_rect(path, rect, inside)\n"
     "--\n\n";
 
-static PyObject *Py_clip_path_to_rect(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_clip_path_to_rect(PyObject *self, PyObject *args)
 {
     py::PathIterator path;
     agg::rect_d rect;
@@ -451,7 +431,7 @@ const char *Py_affine_transform__doc__ =
     "affine_transform(points, trans)\n"
     "--\n\n";
 
-static PyObject *Py_affine_transform(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_affine_transform(PyObject *self, PyObject *args)
 {
     PyObject *vertices_obj;
     agg::trans_affine trans;
@@ -492,7 +472,7 @@ const char *Py_count_bboxes_overlapping_bbox__doc__ =
     "count_bboxes_overlapping_bbox(bbox, bboxes)\n"
     "--\n\n";
 
-static PyObject *Py_count_bboxes_overlapping_bbox(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_count_bboxes_overlapping_bbox(PyObject *self, PyObject *args)
 {
     agg::rect_d bbox;
     numpy::array_view<const double, 3> bboxes;
@@ -634,7 +614,7 @@ const char *Py_cleanup_path__doc__ =
     "return_curves, sketch)\n"
     "--\n\n";
 
-static PyObject *Py_cleanup_path(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_cleanup_path(PyObject *self, PyObject *args)
 {
     py::PathIterator path;
     agg::trans_affine trans;
@@ -717,7 +697,7 @@ const char *Py_convert_to_string__doc__ =
     "--\n\n"
     "Convert *path* to a bytestring.\n"
     "\n"
-    "The first five parameters (up to *sketch*) are interpreted as in \n"
+    "The first five parameters (up to *sketch*) are interpreted as in\n"
     "`.cleanup_path`.  The following ones are detailed below.\n"
     "\n"
     "Parameters\n"
@@ -729,7 +709,7 @@ const char *Py_convert_to_string__doc__ =
     "sketch : tuple of 3 floats, or None\n"
     "precision : int\n"
     "    The precision used to \"%.*f\"-format the values.  Trailing zeros\n"
-    "    and decimal points are always removed.  (precision=-1 is a special \n"
+    "    and decimal points are always removed.  (precision=-1 is a special\n"
     "    case used to implement ttconv-back-compatible conversion.)\n"
     "codes : sequence of 5 bytestrings\n"
     "    The bytes representation of each opcode (MOVETO, LINETO, CURVE3,\n"
@@ -741,7 +721,7 @@ const char *Py_convert_to_string__doc__ =
     "    Whether the opcode comes after the values (True) or before (False).\n"
     ;
 
-static PyObject *Py_convert_to_string(PyObject *self, PyObject *args, PyObject *kwds)
+static PyObject *Py_convert_to_string(PyObject *self, PyObject *args)
 {
     py::PathIterator path;
     agg::trans_affine trans;
@@ -827,54 +807,29 @@ static PyObject *Py_is_sorted(PyObject *self, PyObject *obj)
 
     /* Handle just the most common types here, otherwise coerce to
     double */
-    switch(PyArray_TYPE(array)) {
+    switch (PyArray_TYPE(array)) {
     case NPY_INT:
-        {
-            _is_sorted_int<npy_int> is_sorted;
-            result = is_sorted(array);
-        }
+        result = is_sorted<npy_int>(array);
         break;
-
     case NPY_LONG:
-        {
-            _is_sorted_int<npy_long> is_sorted;
-            result = is_sorted(array);
-        }
+        result = is_sorted<npy_long>(array);
         break;
-
     case NPY_LONGLONG:
-        {
-            _is_sorted_int<npy_longlong> is_sorted;
-            result = is_sorted(array);
-        }
+        result = is_sorted<npy_longlong>(array);
         break;
-
     case NPY_FLOAT:
-        {
-            _is_sorted<npy_float> is_sorted;
-            result = is_sorted(array);
-        }
+        result = is_sorted<npy_float>(array);
         break;
-
     case NPY_DOUBLE:
-        {
-            _is_sorted<npy_double> is_sorted;
-            result = is_sorted(array);
-        }
+        result = is_sorted<npy_double>(array);
         break;
-
     default:
-        {
-            Py_DECREF(array);
-            array = (PyArrayObject *)PyArray_FromObject(obj, NPY_DOUBLE, 1, 1);
-
-            if (array == NULL) {
-                return NULL;
-            }
-
-            _is_sorted<npy_double> is_sorted;
-            result = is_sorted(array);
+        Py_DECREF(array);
+        array = (PyArrayObject *)PyArray_FromObject(obj, NPY_DOUBLE, 1, 1);
+        if (array == NULL) {
+            return NULL;
         }
+        result = is_sorted<npy_double>(array);
     }
 
     Py_DECREF(array);
@@ -910,31 +865,15 @@ static PyMethodDef module_functions[] = {
 };
 
 static struct PyModuleDef moduledef = {
-    PyModuleDef_HEAD_INIT,
-    "_path",
-    NULL,
-    0,
-    module_functions,
-    NULL,
-    NULL,
-    NULL,
-    NULL
+    PyModuleDef_HEAD_INIT, "_path", NULL, 0, module_functions
 };
 
 #pragma GCC visibility push(default)
 
 PyMODINIT_FUNC PyInit__path(void)
 {
-    PyObject *m;
-    m = PyModule_Create(&moduledef);
-
-    if (m == NULL) {
-        return NULL;
-    }
-
     import_array();
-
-    return m;
+    return PyModule_Create(&moduledef);
 }
 
 #pragma GCC visibility pop
