@@ -325,11 +325,11 @@ const char *PyFT2Font_init__doc__ =
     "    Must be positive. Used to scale the hinting in the x-direction\n"
     "_fallback_list : list of FT2Font, optional\n"
     "    A list of FT2Font objects used to find missing glyphs.\n\n"
-    "    .. warning ::\n"
+    "    .. warning::\n"
     "        This API is both private and provisional: do not use it directly\n\n"
     "_kerning_factor : int, optional\n"
     "    Used to adjust the degree of kerning.\n\n"
-    "    .. warning ::\n"
+    "    .. warning::\n"
     "        This API is private: do not use it directly\n\n"
     "Attributes\n"
     "----------\n"
@@ -433,7 +433,7 @@ static int PyFT2Font_init(PyFT2Font *self, PyObject *args, PyObject *kwds)
                || !(data = PyObject_CallMethod(filename, "read", "i", 0))
                || !PyBytes_Check(data)) {
         PyErr_SetString(PyExc_TypeError,
-                        "First argument must be a path or binary-mode file object");
+                        "First argument must be a path to a font file or a binary-mode file object");
         Py_CLEAR(data);
         goto exit;
     } else {
