@@ -20,14 +20,12 @@ is :func:`matplotlib.colors.Normalize`.
 Artists that map data to color pass the arguments *vmin* and *vmax* to
 construct a :func:`matplotlib.colors.Normalize` instance, then call it:
 
-.. ipython::
+.. code-block:: pycon
 
-   In [1]: import matplotlib as mpl
-
-   In [2]: norm = mpl.colors.Normalize(vmin=-1, vmax=1)
-
-   In [3]: norm(0)
-   Out[3]: 0.5
+   >>> import matplotlib as mpl
+   >>> norm = mpl.colors.Normalize(vmin=-1, vmax=1)
+   >>> norm(0)
+   0.5
 
 However, there are sometimes cases where it is useful to map data to
 colormaps in a non-linear fashion.
@@ -53,7 +51,7 @@ N = 100
 X, Y = np.mgrid[-3:3:complex(0, N), -2:2:complex(0, N)]
 
 # A low hump with a spike coming out of the top right.  Needs to have
-# z/colour axis on a log scale so we see both hump and spike.  linear
+# z/colour axis on a log scale, so we see both hump and spike. A linear
 # scale only shows the spike.
 Z1 = np.exp(-X**2 - Y**2)
 Z2 = np.exp(-(X * 10)**2 - (Y * 10)**2)
@@ -192,15 +190,12 @@ plt.show()
 # lower out-of-bounds values to the range over which the colors are
 # distributed. For instance:
 #
-# .. ipython::
+# .. code-block:: pycon
 #
-#   In [2]: import matplotlib.colors as colors
-#
-#   In [3]: bounds = np.array([-0.25, -0.125, 0, 0.5, 1])
-#
-#   In [4]: norm = colors.BoundaryNorm(boundaries=bounds, ncolors=4)
-#
-#   In [5]: print(norm([-0.2, -0.15, -0.02, 0.3, 0.8, 0.99]))
+#   >>> import matplotlib.colors as colors
+#   >>> bounds = np.array([-0.25, -0.125, 0, 0.5, 1])
+#   >>> norm = colors.BoundaryNorm(boundaries=bounds, ncolors=4)
+#   >>> print(norm([-0.2, -0.15, -0.02, 0.3, 0.8, 0.99]))
 #   [0 0 1 2 3 3]
 #
 # Note: Unlike the other norms, this norm returns values from 0 to *ncolors*-1.
@@ -273,11 +268,12 @@ divnorm = colors.TwoSlopeNorm(vmin=-500., vcenter=0, vmax=4000)
 
 pcm = ax.pcolormesh(longitude, latitude, topo, rasterized=True, norm=divnorm,
                     cmap=terrain_map, shading='auto')
-# Simple geographic plot, set aspect ratio beecause distance between lines of
+# Simple geographic plot, set aspect ratio because distance between lines of
 # longitude depends on latitude.
 ax.set_aspect(1 / np.cos(np.deg2rad(49)))
 ax.set_title('TwoSlopeNorm(x)')
-fig.colorbar(pcm, shrink=0.6)
+cb = fig.colorbar(pcm, shrink=0.6)
+cb.set_ticks([-500, 0, 1000, 2000, 3000, 4000])
 plt.show()
 
 
@@ -312,7 +308,8 @@ plt.show()
 # ----------------------------------------------------------
 #
 # The `.TwoSlopeNorm` described above makes a useful example for
-# defining your own norm.
+# defining your own norm.  Note for the colorbar to work, you must
+# define an inverse for your norm:
 
 
 class MidpointNormalize(colors.Normalize):
@@ -323,8 +320,14 @@ class MidpointNormalize(colors.Normalize):
     def __call__(self, value, clip=None):
         # I'm ignoring masked values and all kinds of edge cases to make a
         # simple example...
-        x, y = [self.vmin, self.vcenter, self.vmax], [0, 0.5, 1]
-        return np.ma.masked_array(np.interp(value, x, y))
+        # Note also that we must extrapolate beyond vmin/vmax
+        x, y = [self.vmin, self.vcenter, self.vmax], [0, 0.5, 1.]
+        return np.ma.masked_array(np.interp(value, x, y,
+                                            left=-np.inf, right=np.inf))
+
+    def inverse(self, value):
+        y, x = [self.vmin, self.vcenter, self.vmax], [0, 0.5, 1]
+        return np.interp(value, x, y, left=-np.inf, right=np.inf)
 
 
 fig, ax = plt.subplots()
@@ -334,5 +337,7 @@ pcm = ax.pcolormesh(longitude, latitude, topo, rasterized=True, norm=midnorm,
                     cmap=terrain_map, shading='auto')
 ax.set_aspect(1 / np.cos(np.deg2rad(49)))
 ax.set_title('Custom norm')
-fig.colorbar(pcm, shrink=0.6, extend='both')
+cb = fig.colorbar(pcm, shrink=0.6, extend='both')
+cb.set_ticks([-500, 0, 1000, 2000, 3000, 4000])
+
 plt.show()
