@@ -29,8 +29,8 @@ represent the standard graphical objects we want to paint onto our canvas:
 the containers are places to put them (:class:`~matplotlib.axis.Axis`,
 :class:`~matplotlib.axes.Axes` and :class:`~matplotlib.figure.Figure`).  The
 standard use is to create a :class:`~matplotlib.figure.Figure` instance, use
-the ``Figure`` to create one or more :class:`~matplotlib.axes.Axes` or
-:class:`~matplotlib.axes.Subplot` instances, and use the ``Axes`` instance
+the ``Figure`` to create one or more :class:`~matplotlib.axes.Axes`
+instances, and use the ``Axes`` instance
 helper methods to create the primitives.  In the example below, we create a
 ``Figure`` instance using :func:`matplotlib.pyplot.figure`, which is a
 convenience method for instantiating ``Figure`` instances and connecting them
@@ -59,10 +59,7 @@ graphics primitives (:class:`~matplotlib.lines.Line2D`,
 :class:`~matplotlib.image.AxesImage`, respectively).  These helper methods
 will take your data (e.g., ``numpy`` arrays and strings) and create
 primitive ``Artist`` instances as needed (e.g., ``Line2D``), add them to
-the relevant containers, and draw them when requested.  Most of you
-are probably familiar with the :class:`~matplotlib.axes.Subplot`,
-which is just a special case of an ``Axes`` that lives on a regular
-rows by columns grid of ``Subplot`` instances.  If you want to create
+the relevant containers, and draw them when requested.  If you want to create
 an ``Axes`` at an arbitrary location, simply use the
 :meth:`~matplotlib.figure.Figure.add_axes` method which takes a list
 of ``[left, bottom, width, height]`` values in 0-1 relative figure
@@ -79,8 +76,8 @@ Continuing with our example::
     line, = ax.plot(t, s, color='blue', lw=2)
 
 In this example, ``ax`` is the ``Axes`` instance created by the
-``fig.add_subplot`` call above (remember ``Subplot`` is just a subclass of
-``Axes``) and when you call ``ax.plot``, it creates a ``Line2D`` instance and
+``fig.add_subplot`` call above and when you call ``ax.plot``, it creates a
+``Line2D`` instance and
 adds it to the ``Axes``.  In the interactive `IPython <https://ipython.org/>`_
 session below, you can see that the ``Axes.lines`` list is length one and
 contains the same line that was returned by the ``line, = ax.plot...`` call:
@@ -115,6 +112,7 @@ drawing of the ticks, tick labels and axis labels.
 
 Try creating the figure below.
 """
+# sphinx_gallery_capture_repr = ('__repr__',)
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -122,8 +120,8 @@ import matplotlib.pyplot as plt
 fig = plt.figure()
 fig.subplots_adjust(top=0.8)
 ax1 = fig.add_subplot(211)
-ax1.set_ylabel('volts')
-ax1.set_title('a sine wave')
+ax1.set_ylabel('Voltage [V]')
+ax1.set_title('A sine wave')
 
 t = np.arange(0.0, 1.0, 0.01)
 s = np.sin(2*np.pi*t)
@@ -135,7 +133,7 @@ np.random.seed(19680801)
 ax2 = fig.add_axes([0.15, 0.1, 0.7, 0.3])
 n, bins, patches = ax2.hist(np.random.randn(1000), 50,
                             facecolor='yellow', edgecolor='yellow')
-ax2.set_xlabel('time (s)')
+ax2.set_xlabel('Time [s]')
 
 plt.show()
 
@@ -298,10 +296,10 @@ plt.show()
 #     In [158]: ax2 = fig.add_axes([0.1, 0.1, 0.7, 0.3])
 #
 #     In [159]: ax1
-#     Out[159]: <AxesSubplot:>
+#     Out[159]: <Axes:>
 #
 #     In [160]: print(fig.axes)
-#     [<AxesSubplot:>, <matplotlib.axes._axes.Axes object at 0x7f0768702be0>]
+#     [<Axes:>, <matplotlib.axes._axes.Axes object at 0x7f0768702be0>]
 #
 # Because the figure maintains the concept of the "current Axes" (see
 # :meth:`Figure.gca <matplotlib.figure.Figure.gca>` and
@@ -348,7 +346,7 @@ plt.show()
 # ================ ============================================================
 # Figure attribute Description
 # ================ ============================================================
-# axes             A list of `~.axes.Axes` instances (includes Subplot)
+# axes             A list of `~.axes.Axes` instances
 # patch            The `.Rectangle` background
 # images           A list of `.FigureImage` patches -
 #                  useful for raw pixel display
@@ -558,7 +556,7 @@ plt.show()
 # :attr:`~matplotlib.axes.Axes.yaxis`.  The ``XAxis`` and ``YAxis``
 # containers will be detailed below, but note that the ``Axes`` contains
 # many helper methods which forward calls on to the
-# :class:`~matplotlib.axis.Axis` instances so you often do not need to
+# :class:`~matplotlib.axis.Axis` instances, so you often do not need to
 # work with them directly unless you want to.  For example, you can set
 # the font color of the ``XAxis`` ticklabels using the ``Axes`` helper
 # method::
@@ -718,6 +716,6 @@ plt.show()
 # dollar signs and colors them green on the right side of the yaxis.
 #
 #
-# .. include:: ../../gallery/pyplots/dollar_ticks.rst
-#    :start-after: y axis labels.
+# .. include:: ../../gallery/ticks/dollar_ticks.rst
+#    :start-after: .. redirect-from:: /gallery/pyplots/dollar_ticks
 #    :end-before: .. admonition:: References
